// kio_z3950r.h
//
// Author:    Robby Stephenson
// Email:     robby@periapsis.org
// Copyright: 2002
// License:   GNU General Public License, version 2

// Requires the yaz and yaz++ libraries, http://www.indexdata.dk/yaz/

#ifndef KIO_ZOOM_H
#define KIO_ZOOM_H

#include <kurl.h>
#include <kio/global.h>
#include <kio/slavebase.h>

#include <qcstring.h>

/**
 * This KIOSlave implements a z39.50 Retrieval URL, using the ZOOM
 * API. It follows RFC 2056, except that the zscheme
 * is z3950r instead of z39.50r.
 *
 * (excerpt from rfc2056)
 *
 * 4. The Z39.50 Retrieval URL
 *
 * The Z39.50 Retrieval URL is intended to allow a Z39.50 session to be
 * used as a transparent transfer mechanism to retrieve a specific
 * information object.  A Z39.50 client uses information in the URL to
 * formulate a Search Request.  The server's Search Response indicates
 * how many records match the Request.  If the number of matching
 * records does not equal one, the retrieval is considered unsuccessful,
 * and the client application's behavior is not defined.  If the number
 * of matching records equals one, the server may have included the
 * desired record in the Search Response.  If not, the client requests
 * transmission of the record with a Present Request.  After the client
 * has received the specified record it may close the Z39.50 session
 * immediately, or keep it open for subsequent retrievals.

 * -  Host is required.
 * -  Port is optional, and defaults to 210.
 * -  A database is required.
 * -  The meaning of a retrieval URL with no docid is undefined.
 * -  The docid is placed into a type-1 query, as the single term, in
 *    the general format (tag 45), using the Bib-1 attribute set, with
 *    a Use attribute value of docid, and a structure attribute of URx.
 *    The docid string is server-defined and completely opaque to the
 *    client.
 * - If element set name (esn) is not specified, it is the client's
 *    choice.  If esn is specified, it should be used either in the
 *    Search request for the value of small- and/or medium-
 *    set-element-set-names or in a Present request following a
 *    Search.  These terms and their use are defined within the Z39.50
 *    Standard [2].
 * -  If record syntax (rs) is not specified, it is the client's choice.
 *    If one or more record syntaxes are specified, the client should
 *    select one (preferably the first in the list that it supports)
 *    and use it in a Search or Present request as the value of
 *    PreferredRecordSyntax.
 *
 * 5. BNF for Z39.50 URLs
 *
 * The Z39.50 Session and Retrieval URLs follow the Common Internet
 * Scheme Syntax as defined in RFC 1738, "Uniform Resource Locators
 * (URL)" [1].  In the definition, literals are quoted with "", optional
 * elements are enclosed in [brackets], "|" is used to designate
 * alternatives, and elements may be preceded with <n>* to designate n
 * or more repetitions of the following element; n defaults to 0.
 *
 * z39.50url      = zscheme "://" host [":" port]
 *                           ["/" [database *["+" database]
 *                           ["?" docid]]
 *                           [";esn=" elementset]
 *                           [";rs=" recordsyntax *[ "+" recordsyntax]]]
 *
 * zscheme        = "z39.50r" | "z39.50s"
 * database       = uchar
 * docid          = uchar
 * elementset     = uchar
 * recordsyntax   = uchar
 */
class Z3950rProtocol : public KIO::SlaveBase {

public:
  Z3950rProtocol(const QCString& pool_socket, const QCString& app_socket);
  virtual void get(const KURL& url);
};

#endif
